@@ -89,6 +89,12 @@ try:
 
         # float changed to half to save memory
         torch_T_to_np = lambda T: T.to("cpu").half().numpy()
+        EPSILON = HALF_EPSILON
+        EVALS_THRESH = EVALS_HALF_THRESH
+        
+        logger.info(f"Using EPSILON = {EPSILON}")
+        logger.info(f"Using EVALS_THRESH = {EVALS_THRESH}")
+
          
         def torch_wrapper(M, f):
             torch.cuda.empty_cache()
@@ -176,16 +182,27 @@ def calc_rank_loss(svals, N, tol=None):
 def matrix_entropy(svals, N):
     """Matrix entropy of real, computed using the singular values, and the dim N"""
 
-
-    rank = matrix_rank(svals, N) #np.linalg.matrix_rank(W)
-
-    evals = svals*svals
-    p = evals / np.sum(evals)
-    if (rank == 1):
-        rank = 1.000001
-    entropy = -np.sum(p * np.log(p)) / np.log(rank) 
+    entropy = -1
+    
+    try:
+    
+        rank = matrix_rank(svals, N) #np.linalg.matrix_rank(W)
+    
+        evals = svals*svals
+        p = evals / np.sum(evals) + EPSILON
+        rank += EPSILON
+        entropy = -np.sum(p * np.log(p)) / np.log(rank) 
+        
+    except (ZeroDivisionError, ValueError) as e:
+        # Handle divide by zero and invalid value errors
+        logger.warning("Error:", e)
+    except Exception as e:
+        # Handle other fatal errors
+        logger.warning("Error:", e)
+    
     return entropy
 
+
 # Wigner SemiCircle Plots
 
 
@@ -453,13 +470,13 @@ def discrete_entropy(vec, num_bins=100):
     vec = np.absolute(vec)
     vec = vec - np.mean(vec)
     h = np.histogram(vec, density=True, bins=num_bins)[0];
-    p = np.array(h) + 0.0000000001
+    p = np.array(h) + EPSILON
     
     p = p / np.sqrt(2 * np.pi)
     p = p / np.sum(p)
 
     # p = p/(2*np.pi)
-    entropy = -np.sum(p * np.log(p))
+    entropy = -np.sum(p * np.log(o))
     entropy = entropy  # /(2*np.pi)#/float(num_bins)
     return entropy
 
@@ -984,7 +1001,7 @@ def combine_weights_and_biases(W,b):
     """concat the biases onto the weight matrix if not null or zero"""
     
     Wb = W
-    if b is not None and np.sum(b)>ALMOST_ZERO:
+    if b is not None and np.sum(b)>EPSILON:
         if W.shape[1]==b.shape[0]:
             Wb = np.vstack([W,b])
         else:
